% Matlab codes associated with manuscript, entitled "Technical Note:
% Monitoring of unsteady open channel flows using the continuous slope-area method" by Kyutae Lee et al (doi:10.5194/hess-21-1-2017). 

% More information about the modified fread method can be found at
% http://ascelibrary.org/doi/10.1061/%28ASCE%29HY.1943-7900.0001280.

% Please contact Kyutae lee at this email address: alkene78@gmail.com for
% further questions.

%% The first channel conveyance at upstream PT (Eq.(3)).
clear all
%read cross sections at the upstream PT.
%(x=distance; y=elevation).
x=xlsread('Data.xlsx', 'Cross_Section', 'A3:A19');
y=xlsread('Data.xlsx', 'Cross_Section', 'B3:B19');

% Linear interpolation between surveyed points. 
ns=500000;
xi = linspace(0,x(end),ns)';
yi = interp1(x,y,xi,'linear');

%calculate cross sectional areas(a) by changing stages(s)
limit=216.6; 
h=min(y):0.01:limit;

for i=1:length(h);
    y_new=yi; %input the original data for each loop
    y_new(y_new>=h(i))=h(i);%filter the stages
    a(i)=max(x)*max(y_new)-trapz(xi,y_new); % calculate the area at each stage
end

%read the excel file containing recorded stages at upstream PT for the selected event. 
%s_time1=xlsread('Data.xlsx', 'Data','E4538:E4725');
%s_time1=xlsread('Data.xlsx', 'Data','E870:E1014');
s_time1=xlsread('Data.xlsx', 'Data','E2:E198');
a_time1=interp1(h,a,s_time1); %interpolate areas corresponding to recorded stages in time-series.

%calculate perimeters (total) at each stage
for i=1:length(h);
    x_new=xi;
    y_new=yi; %input the original data for each loop
    x_real=x_new(y_new<=h(i)); %filter the stages
    y_real=y_new(y_new<=h(i));
    
    
    x_d=diff(x_real);
    y_d=diff(y_real);
        
    if x_d<(x(end)/(ns-2));
        seg=sqrt(x_d.^2+y_d.^2);
        total(i)=sum(seg);
        
    else 
        x_s1=x_real(x_d > (x(end)/(ns-2)));
        y_s1=y_real(x_d > (x(end)/(ns-2)));
        x_s2=x_real(find(x_d > (x(end)/(ns-2)))+1);
        y_s2=y_real(find(x_d > (x(end)/(ns-2)))+1);
        seg=sqrt(x_d.^2+y_d.^2);
        disc=sqrt((x_s2-x_s1).^2+(y_s2-y_s1).^2);
        total(i)=sum(seg)-sum(disc);
    end
   
end

p_time=interp1(h,total,s_time1); %calculate perimeters corresponding to recorded stages in time-series at upstream PT.
R1=a_time1./p_time; %calculate hydraulic radius in time-series at upstream PT.

% reset variables
clearvars a total h p_time 

%% The second channel conveyance at downstream PT (Eq.(3)).
%read cross sections at the downstream PT.
%(x=distance; y=elevation).
x=xlsread('Data.xlsx', 'Cross_Section', 'C2:C12');
y=xlsread('Data.xlsx', 'Cross_Section', 'D2:D12');

ns=500000;
xi = linspace(0,x(end),ns)';
yi = interp1(x,y,xi,'linear');

%calculate areas(a) by changing stages(s)
limit=216.6; 
h=min(y):0.01:limit;

for i=1:length(h);
    y_new=yi; %input the original data for each loop
    y_new(y_new>=h(i))=h(i);%filter the stages
    a(i)=max(x)*max(y_new)-trapz(xi,y_new); % calculate the area for each stage
end

%read the excel file containing recorded stages at downstream PT for the selected event.
%s_time=xlsread('Data.xlsx', 'Data','F4538:F4725');
%s_time=xlsread('Data.xlsx', 'Data','F870:F1014');
s_time=xlsread('Data.xlsx', 'Data','F2:F198');
a_time=interp1(h,a,s_time); %interpolate areas corresponding to recorded stages in time-series.

%calculate perimeters (total) at each stage
for i=1:length(h);
    x_new=xi;
    y_new=yi; %input the original data for each loop
    x_real=x_new(y_new<=h(i)); %filter the stages
    y_real=y_new(y_new<=h(i));
    
    
    x_d=diff(x_real);
    y_d=diff(y_real);
        
    if x_d<(x(end)/(ns-2));
        seg=sqrt(x_d.^2+y_d.^2);
        total(i)=sum(seg);
        
    else 
        x_s1=x_real(x_d > (x(end)/(ns-2)));
        y_s1=y_real(x_d > (x(end)/(ns-2)));
        x_s2=x_real(find(x_d > (x(end)/(ns-2)))+1);
        y_s2=y_real(find(x_d > (x(end)/(ns-2)))+1);
        seg=sqrt(x_d.^2+y_d.^2);
        disc=sqrt((x_s2-x_s1).^2+(y_s2-y_s1).^2);
        total(i)=sum(seg)-sum(disc);
    end
   
end

p_time=interp1(h,total,s_time); %calculate perimeters corresponding to recorded stages in time-series at downstream PT.
R=a_time./p_time; %calculate hydraulic radius in time-series at downstream PT.
    
%% Read USGS discharge and time records
%read the excel file containing discharges and date/time at USGS gauge station for the selected event.

%USGS_Q=xlsread('Data.xlsx', 'Data','D4538:D4725');
%USGS_Q=xlsread('Data.xlsx', 'Data','D870:D1014');
USGS_Q=xlsread('Data.xlsx', 'Data','D2:D198');

%date=xlsread('Data.xlsx', 'Data','B4538:B4725');
%date=xlsread('Data.xlsx', 'Data','B870:B1014');
date=xlsread('Data.xlsx', 'Data','B2:B198');
date2=datestr(x2mdate(date)); %Convert date/time format

%% Evaluate the discharge hydrograph based on Manning's equation.
V1=(USGS_Q)./(a_time1); %Estimate velocities at upstream PT.
V=(USGS_Q)./(a_time); %Estimate velocities at downstream PT.
constant_slope=0.00039; %measured channel slope
manning_n1=(sqrt(constant_slope).*(R1.^(2/3)))./(V1); % Manning's roughness coefficcients at upstream PT
manning_n2=(sqrt(constant_slope).*(R.^(2/3)))./(V); % Manning's roughness coefficcients at downstream PT
manning_bray=0.104*(constant_slope.^0.177); % Eq.(6)
manning_jarrett=0.39*(constant_slope.^0.38).*(R.^-0.16); % Eq.(7)
manning_sauer=0.11*(constant_slope.^0.18).*(R.^(0.08)); % Eq.(8)

% Read measured water slopes by a pair of installed pressure transducers. 
% The data is moving averaged (1hour 30mins)to reduce noise. 
% slope=xlsread('Data.xlsx', 'Data','G4538:G4725');
% slope=xlsread('Data.xlsx', 'Data','G870:G1014');
slope=xlsread('Data.xlsx', 'Data','G2:G198');

Q_CSA=sqrt(((1./manning_n1).*a_time1.*((R1).^(2/3))).*((1./manning_n2).*a_time.*((R).^(2/3))).*slope);
Q_bray=(1./manning_bray).*a_time.*((R).^(2/3)).*((slope).^(1/2));
Q_jarrett=(1./manning_jarrett).*a_time.*((R).^(2/3)).*((slope).^(1/2));
Q_sauer=(1./manning_sauer).*a_time.*((R).^(2/3)).*((slope).^(1/2));

%% Free surface width (T) at downstream PT

for i=2:length(h);
    x_new=xi;
    y_new=yi; %input the original data for each loop
    x_real=x_new(y_new<=h(i)); %filter the stages
    
    x_d=diff(x_real);

    if x_d<(x(end)/(ns-2));
        T(i)=x_real(end)-x_real(1);
    else
        x_s1=x_real(x_d > (x(end)/(ns-2)));
        x_s2=x_real(find(x_d > (x(end)/(ns-2)))+1);
        disc=sum(x_s2-x_s1);
        T(i)=x_real(end)-x_real(1)-sum(disc);
    end
end

T_time=interp1(h,T,s_time); % Interpolate free surface width (T) based on time-series

%% Modified Fread formula 
g= 9.81; %gravity force
dhdt=900; %15mins data interval = 900s

%date_new=xlsread('Data.xlsx', 'Data','B4539:B4725');
%date_new=xlsread('Data.xlsx', 'Data','B871:B1014');
date_new=xlsread('Data.xlsx', 'Data','B3:B198'); %new date by excluding the first data. 
date3=datestr(x2mdate(date_new));

% Wave celerity coefficient K 
K_fread_m=(5/3)-(2*a(2:end)./(3*total(2:end))).*(diff(total)./diff(a));
K_fread1_m=interp1(h(2:end),K_fread_m,s_time); %interpolation 

% Implicit equation below can be solved iteratively to estimate unsteady discharges. 
syms Q Real;
Q0=USGS_Q(1);
for i=1:length(USGS_Q)-1;
    K(i)=double(solve(Q-(((a_time(i+1)).*(R(i+1).^(2/3)))./manning_n2(i+1))...
        .*sqrt(constant_slope+(a_time(i+1)./(K_fread1_m(i).*Q)).*(diff(s_time(i:i+1))/dhdt)...
        +(Q./(g*(a_time(i+1).^2))).*((diff(a_time(i:i+1))/dhdt)-((T_time(i+1)./(K_fread1_m(i))).*(diff(s_time(i:i+1))/dhdt)))...
        +(Q0./a_time(i)-Q./a_time(i+1))/(g*dhdt))...
        ,Q,'PrincipalValue',true));
    A(i)=(((a_time(i+1)).*(R(i+1).^(2/3)))./manning_n2(i+1));
    b(i)=constant_slope;
    c(i)=(a_time(i+1)./(K_fread1_m(i).*K(i))).*(diff(s_time(i:i+1))/dhdt);
    d(i)=(K(i)./(g*(a_time(i+1).^2))).*((diff(a_time(i:i+1))/dhdt)-((T_time(i+1)./(K_fread1_m(i))).*(diff(s_time(i:i+1))/dhdt)));
    e(i)=(Q0./a_time(i)-K(i)./a_time(i+1))/(g*dhdt);
    Q0=K(i);
end

Q_modified_Fread=real(A.*((b+c+d+e).^(1/2))); %Computed unsteady discharges based on the modified Fread method

%% Result 1 (stage-discharge rating curve)

% Create figure
figure1 = figure;
% Create plot
plot(USGS_Q, s_time,'LineWidth',2,'Color',[0 0 0],'DisplayName','USGS'); hold on;
plot(Q_CSA,s_time,'LineWidth',2,'Color',[1 0 0],'DisplayName','CSA'); hold on;
plot(Q_modified_Fread, s_time(2:end), 'LineWidth',2,'Color',[0 0 1],'DisplayName','Modified Fread'); hold on;
% plot(Q_bray, s_time, 'LineWidth',2,'Color',[0 1 1],'DisplayName','Q_bray'); hold on;
% plot(Q_jarrett, s_time, 'LineWidth',2,'Color',[1 0 1],'DisplayName','Q_jarrett'); hold on;
% plot(Q_sauer, s_time, 'LineWidth',2,'Color',[0 1 0],'DisplayName','Q_sauer'); hold on;

grid on;
% Create xlabel
xlabel('Q (cms)');
% Create ylabel
ylabel('Stage (m)');
% Create title
% title({'Stage - Discharge Rating Curve'},'FontSize',14);
% Create legend
legend('show');

%% Result 2 (discharge time-series)

ts1=timeseries(Q_modified_Fread,date3);
ts1.TimeInfo.Format = 'dd-mmm-yyyy';
ts2=timeseries(USGS_Q,date2);
ts2.TimeInfo.Format = 'dd-mmm-yy hh:00';
ts3=timeseries(Q_CSA,date2);
ts3.TimeInfo.Format = 'dd-mmm-yy hh:00';
% ts4=timeseries(Q_bray,date2);
% ts4.TimeInfo.Format = 'dd-mmm-yy hh:00';
% ts5=timeseries(Q_jarrett,date2);
% ts5.TimeInfo.Format = 'dd-mmm-yy hh:00';
% ts6=timeseries(Q_sauer,date2);
% ts6.TimeInfo.Format = 'dd-mmm-yy hh:00';

figure2=figure;
plot(ts1,'-b','DisplayName','modified Fread Q'); hold on;
plot(ts2,'-g','DisplayName','USGS Q'); hold on;
plot(ts3,'-r','DisplayName','CSA Q'); hold on;
% plot(ts4,'-c','DisplayName','Bray Q'); hold on;
% plot(ts5,'--b','DisplayName','Jarret Q'); hold on;
% plot(ts6,'--g','DisplayName','Sauer Q'); hold on;
grid on;
% Create xlabel
xlabel({' Time Series'},'Interpreter','none');
% Create ylabel
ylabel({' Q'},'Interpreter','none');
% Create legend
legend('show');

%% Result 3 (Manning's roughness coefficients)

figure3=figure;
plot(manning_n1, s_time, '-b','DisplayName','Upstream PT'); hold on;
plot(manning_n2,s_time,'-g','DisplayName','Downstream PT'); hold on;
plot(0.5.*(manning_n1+manning_n2),s_time,'-r','DisplayName','Average PT'); hold on; %Average for visualization
plot(manning_bray.*ones(size(s_time)),s_time,'-c','DisplayName','Bray'); hold on;
plot(manning_jarrett,s_time,'--b','DisplayName','Jarret'); hold on;
plot(manning_sauer,s_time,'--g','DisplayName','Sauer'); hold on;
grid on;
% Create xlabel
xlabel({' Time Series'},'Interpreter','none');
% Create ylabel
ylabel({' Q'},'Interpreter','none');
% Create legend
legend('show');
